Library Arduino;

// Arduino Driver for WinFACT (all versions)
// DELHPI 6

{$R *.res}

uses
  WinProcs,
  WinTypes,
  SysUtils,
  Controls,
  Forms,
  Classes,
  Graphics,
  StdCtrls,
  Dialogs,
  Sockets,
  NMUDP, Math,
  ArduinoDlg in 'ArduinoDlg.pas' {ArduinoDialog};

type
  TDummy = class  // Dummy Class to receive UDP Data
    OwnerUserData: Pointer;
    procedure DataReceived(Sender: TComponent; NumberBytes: Integer; FromIP: string; Port: integer);
  end;

  PUserData = ^TUserData;
  TUserData = record
    Dummy: TDummy;          // Dummy Class to receive UDP Data
    DataReceived: boolean;  // Data received Flag
    ReceivedString: string; // Received String
    UDPClient: TNMUDP;      // Client for UDP Communication
    // Configuration data
    Mode: integer;        // Interface (0: Ethernet, 1: USB)
    COMPort: integer;     // COM-Port
    Baudrate: integer;    // Baud rate for COM-Port
    RemotePort: integer;  // Remote-Port for UDP
    IPAddress: string;    // IP-Address of Arduino
    DigPinMode: array[2..9] of integer;  // Mode of Pin 2...9 (0: unused, 1: Digital Input, 2: Digital Output, 3: PWM Output)
    AnaPinMode: array[0..5] of integer;  // Mode of Pin A0...A5 (0: unused, 1: used) 
  end;

  PParameterStruct=^TParameterStruct;
  TParameterStruct=packed record
    NuE : Byte;
    NuI : Byte;
    NuB : Byte;
    E:Array[0..31] of Extended;
    I:Array[0..31] of Integer;
    B:Array[0..31] of Byte;
    D:Array[0..255] of char;
    EMin:Array[0..31] of Extended;
    EMax:Array[0..31] of Extended;
    IMin:Array[0..31] of Integer;
    IMax:Array[0..31] of Integer;
    NaE : Array[0..31,0..40] of char;
    NaI : Array[0..31,0..40] of char;
    NaB : Array[0..31,0..40] of char;
    UserDataPtr: PUserData;
    ParentPtr: Pointer;
    ParentHWnd: HWnd;
    ParentName: PChar;
    UserHWindow: HWnd;
    DataFile: text;
  end;

  PDialogEnableStruct=^TDialogEnableStruct;
  TDialogEnablestruct=packed record
    AllowE: Longint;
    AllowI: Longint;
    AllowB: Longint;
    AllowD: Byte;
  end;

  PNumberOfInputsOutputs=^TNumberOfInputsOutputs;
  TNumberOfInputsOutputs=packed record
    Inputs :Byte;
    Outputs:Byte;
    NameI : Array[0..49,0..40] of char;
    NameO : Array[0..49,0..40] of char;
  end;

  PInputArray = ^TInputArray;
  TInputArray = packed array[1..30] of extended;
  POutputArray = ^TOutputArray;
  TOutputArray = packed array[1..30] of extended;

var
  COMHandle: THandle;

procedure GetParameterStruct(D:PParameterStruct);export stdcall;
begin
  D^.NuE := 0;
  D^.NuI := 1;
  D^.NuB := 0;
  D^.I[0] := 0;
end;

procedure GetDialogEnableStruct(D:PDialogEnableStruct;D2:PParameterStruct);export stdcall;
begin
  D^.AllowE:=$FFFFFFFF;
  D^.AllowB:=$FFFFFFFF;
  D^.AllowI:=$FFFFFFFF;
  D^.AllowD:= 1;
end;

procedure GetNumberOfInputsOutputs(D:PNumberOfInputsOutputs);export stdcall;
begin
  D^.Outputs := 8 + 6;
  D^.Inputs := 8;
  // Outputs
  StrPCopy(D^.NameO[0], 'Pin2');
  StrPCopy(D^.NameO[1], 'Pin3');
  StrPCopy(D^.NameO[2], 'Pin4');
  StrPCopy(D^.NameO[3], 'Pin5');
  StrPCopy(D^.NameO[4], 'Pin6');
  StrPCopy(D^.NameO[5], 'Pin7');
  StrPCopy(D^.NameO[6], 'Pin8');
  StrPCopy(D^.NameO[7], 'Pin9');
  StrPCopy(D^.NameO[8], 'Analog Input A0');
  StrPCopy(D^.NameO[9], 'Analog Input A1');
  StrPCopy(D^.NameO[10], 'Analog Input A2');
  StrPCopy(D^.NameO[11], 'Analog Input A3');
  StrPCopy(D^.NameO[12], 'Analog Input A4');
  StrPCopy(D^.NameO[13], 'Analog Input A5');
  // Inputs
  StrPCopy(D^.NameI[0], 'Pin2');
  StrPCopy(D^.NameI[1], 'Pin3');
  StrPCopy(D^.NameI[2], 'Pin4');
  StrPCopy(D^.NameI[3], 'Pin5');
  StrPCopy(D^.NameI[4], 'Pin6');
  StrPCopy(D^.NameI[5], 'Pin7');
  StrPCopy(D^.NameI[6], 'Pin8');
  StrPCopy(D^.NameI[7], 'Pin9');
end;

function SetOutputChar: PChar; export stdcall;
begin
  Result := '23456789A&0A&1A&2A&3A&4A&5';
end;

function SetInputChar: PChar; export stdcall;
begin
  Result := '23456789';
end;

procedure TDummy.DataReceived(Sender: TComponent; NumberBytes: Integer; FromIP: string; Port: integer);
var Buffer: array[0..300] of char;
    nReadBytes: integer;
begin
  FillChar(Buffer, sizeof(Buffer), 0);
  with PUserData(OwnerUserData)^ do begin
    UDPClient.ReadBuffer(Buffer, nReadBytes);
    ReceivedString := StrPas(Buffer);
    DataReceived := true;
  end;
end;

procedure SendString(D: PParameterStruct; Command: string);
// Send String
var Buffer: array[0..100] of char;
    BytesWritten: DWORD;
    i: integer;
begin
  if D^.UserDataPtr.Mode = 0 then begin  // Ethernet
    StrPCopy(Buffer, Command);
    D^.UserDataPtr.UDPClient.SendBuffer(Buffer, Length(Command));
  end else begin  // USB
    for i:=1 to Length(Command) do
      WriteFile(ComHandle, Command[i], 1, BytesWritten, nil);
  end;
end;

function ReadString(D: PParameterStruct): string;
// Read String
var i, StartTime: cardinal;
    BytesRead: DWORD;
    InBytes: array[1..1] of byte;
begin
  StartTime := GetTickCount;
  if D^.UserDataPtr.Mode = 0 then begin  // Ethernet
    D^.UserDataPtr.DataReceived := false;
    D^.UserDataPtr.ReceivedString := '';
    repeat
      Application.ProcessMessages;
    until D^.UserDataPtr.DataReceived or ((GetTickCount - StartTime) > 100);
    Result := D^.UserDataPtr.ReceivedString;
  end else begin  // USB
    Result := '';
    repeat
      InBytes[1] := 0;
      ReadFile(ComHandle, InBytes, 1, BytesRead, nil);
      if BytesRead > 0 then if InBytes[1] > 13 then
        Result := Result + chr(InBytes[1]);
      Application.ProcessMessages;
    until (InBytes[1] = 10) or ((GetTickCount - StartTime) > 1000);
  end;
end;

function CanSimulateDLL(D:PParameterStruct):Integer; export stdcall;
// Called to check if simulation is possible
var ComHandle: THandle;
begin
  Screen.Cursor := crHourGlass;
  // Ethernet
  if D^.UserDataPtr.Mode = 0 then begin
    Result := 1;
  end;
  // RS-232
  if D^.UserDataPtr.Mode = 1 then begin
    Result := 0;
    if D^.UserDataPtr.ComPort < 10 then
      ComHandle := CreateFile(PChar('COM' + IntToStr(D^.UserDataPtr.ComPort)),GENERIC_READ or GENERIC_WRITE,0,nil,OPEN_EXISTING,0,0)
    else
      ComHandle := CreateFile(PChar('\\.\COM' + IntToStr(D^.UserDataPtr.ComPort)),GENERIC_READ or GENERIC_WRITE,0,nil,OPEN_EXISTING,0,0);
    if(ComHandle <> INVALID_HANDLE_VALUE) then begin
      Result := 1;
      CloseHandle(ComHandle);
      ComHandle := 0;
    end;
  end;
  Screen.Cursor := crDefault;
end;

procedure SimulateDLL(T:Extended;D1:PParameterStruct;Inputs:PInputArray;Outputs:POutputArray);export stdcall;
// Called for each simulation step
var
  i: integer;
  s: string;
begin
  // Digital I/Os and PWM Outputs
  for i:=2 to 9 do if D1^.UserDataPtr.DigPinMode[i] > 0 then begin
    case D1^.UserDataPtr.DigPinMode[i] of
      1: begin // Digital Input
           s := '?' + Format('%2.2d', [i]) + 's?';
           SendString(D1, s);
           s := ReadString(D1);
           if s = 'LOW' then
             Outputs^[i-1] := 0
           else
             Outputs^[i-1] := 5;
         end;
      2: begin // Digital Output
           if Inputs^[i-1] > 2.5 then
             s := '!' + Format('%2.2d', [i]) + 's1'
           else
             s := '!' + Format('%2.2d', [i]) + 's0';
           SendString(D1, s);
           s := ReadString(D1); // Echo lesen
         end;
      3: begin // PWM Output
           Inputs^[i-1] := Min(Max(Inputs^[i-1], 0), 1023);
           s := Format('%4.4d', [round(Inputs^[i-1])]);
           SendString(D1, 'pwm' + Format('%2.2d', [i]));
           ReadString(D1);  // Read echo
           SendString(D1, 'w' + s);  // formatted value
           ReadString(D1);  // Read echo
         end;
    end;
  end;
  // Analog Inputs
  for i:=0 to 5 do if D1^.UserDataPtr.AnaPinMode[i] > 0 then begin
    SendString(D1, '?' + Format('%2.2d', [i+14]) + 's?');
    s := ReadString(D1);
    Outputs^[i+9] := StrToFloat(s);
  end;
end;

procedure InitSimulationDLL(D:PParameterStruct;Inputs:PInputArray;Outputs:POutputArray);export stdcall;
// Called when simulation starts
var DCB: TDCB;
    CommTimeOuts: TCommTimeOuts;
    s: string;
    i: integer;
begin
  if D^.UserDataPtr.Mode = 0 then begin  // Ethernet
    with D^.UserDataPtr^ do begin
      UDPClient.RemoteHost := IPAddress;
      UDPClient.RemotePort := RemotePort;
    end;
  end;
  if D^.UserDataPtr.Mode = 1 then begin  // USB
    if ComHandle = 0 then begin
      if D^.UserDataPtr.ComPort < 10 then
        ComHandle := CreateFile(PChar('COM' + IntToStr(D^.UserDataPtr.ComPort)),GENERIC_READ or GENERIC_WRITE,0,nil,OPEN_EXISTING,0,0)
      else
        ComHandle := CreateFile(PChar('\\.\COM' + IntToStr(D^.UserDataPtr.ComPort)),GENERIC_READ or GENERIC_WRITE,0,nil,OPEN_EXISTING,0,0);
      DCB.DCBlength := SizeOf(DCB);
      GetCommState(ComHandle, DCB);
      DCB.BaudRate := D^.UserDataPtr.Baudrate;
      DCB.ByteSize := 8;
      DCB.Parity := 0;
      DCB.StopBits := ONESTOPBIT;
      DCB.Flags := 1;
      SetCommState(ComHandle, DCB);
      GetCommTimeOuts(ComHandle, CommTimeOuts);
      CommTimeOuts.ReadTotalTimeoutConstant := 10;
      SetCommTimeOuts(ComHandle, CommTimeOuts);
      sleep(2000);
    end;
  end;
  // ---- Configure inputs/outputs
  for i:=2 to 9 do if D^.UserDataPtr.DigPinMode[i] > 0 then begin  // Pin 2 ... 9
    case D^.UserDataPtr.DigPinMode[i] of
      1:    s := '!' + Format('%2.2d', [i]) + 'mi';
      2, 3: s := '!' + Format('%2.2d', [i]) + 'mo';
    end;
    SendString(D, s);
    s := ReadString(D);  // Read echo
  end;
  for i:=14 to 19 do if D^.UserDataPtr.AnaPinMode[i-14] > 0 then begin  // Pin 14 ... 19
    SendString(D, '!' + Format('%2.2d', [i]) + 'mi');
    ReadString(D); // Read echo
  end;  
  // ----------------------------
  SimulateDLL(0, D, Inputs, Outputs);
end;

procedure EndSimulationDLL2(D:PParameterStruct); export stdcall;
// Called when simulation was terminated
begin
  // Ethernet
  if D^.UserDataPtr.Mode = 0 then begin
  end;
  // RS-232
  if D^.UserDataPtr.Mode = 1 then begin
    if ComHandle <> 0 then
      CloseHandle(ComHandle);
    ComHandle := 0;
  end;
end;

procedure InitUserDLL(D: PParameterStruct); export stdcall;
var i: integer;
begin
  Application.Handle := D^.ParentHWnd;
  D^.UserDataPtr := new(PUserData);
  with D^.UserDataPtr^ do begin
    Mode := 0; // Ethernet
    COMPort := 1;
    Baudrate := 9600;
    IPAddress := '192.168.0.121';
    RemotePort := 1600;
    for i:=2 to 5 do
      DigPinMode[i] := 1;  // Digital Input
    for i:=6 to 9 do
      DigPinMode[i] := 2;  // DigitalOutput
    for i:=0 to 5 do
      AnaPinMode[i] := 1;  // used
    // UDP Client
    Dummy := TDummy.Create;
    Dummy.OwnerUserData := D^.UserDataPtr;
    UDPClient := TNMUDP.Create(nil);
    UDPClient.OnDataReceived := Dummy.DataReceived;
    UDPClient.LocalPort := 1600;
  end;
end;

procedure DisposeUserDLL(D: PParameterStruct); export stdcall;
begin
  Application.Handle := 0;
  D^.UserDataPtr.UDPClient.Free;
  Dispose(D^.UserDataPtr);
end;

function GetDLLName: PChar; export stdcall;
begin
  GetDLLName := 'Arduino';
end;

procedure CallParameterDialogDLL(D1:PParameterStruct; D2:PNumberOfInputsOutputs);export stdcall;
var Dialog: TArduinoDialog;
begin
  Dialog := TArduinoDialog.Create(Application);
  with Dialog, D1^.UserDataPtr^ do begin
    RadioButton1.Checked := Mode = 0;
    RadioButton2.Checked := Mode = 1;
    IPAddressEdit.Text := IPAddress;
    PortEdit.Text := IntToStr(RemotePort);
    UpDown1.Position := COMPort;
    case Baudrate of
      9600:   BaudCombobox.ItemIndex := 0;
      19200:  BaudCombobox.ItemIndex := 1;
      38400:  BaudCombobox.ItemIndex := 2;
      115200: BaudCombobox.ItemIndex := 3;
    end;
    P2Combobox.ItemIndex := DigPinMode[2];
    P3Combobox.ItemIndex := DigPinMode[3];
    P4Combobox.ItemIndex := DigPinMode[4];
    P5Combobox.ItemIndex := DigPinMode[5];
    P6Combobox.ItemIndex := DigPinMode[6];
    P7Combobox.ItemIndex := DigPinMode[7];
    P8Combobox.ItemIndex := DigPinMode[8];
    P9Combobox.ItemIndex := DigPinMode[9];
    A0Combobox.ItemIndex := AnaPinMode[0];
    A1Combobox.ItemIndex := AnaPinMode[1];
    A2Combobox.ItemIndex := AnaPinMode[2];
    A3Combobox.ItemIndex := AnaPinMode[3];
    A4Combobox.ItemIndex := AnaPinMode[4];
    A5Combobox.ItemIndex := AnaPinMode[5];
    if ShowModal = mrOK then begin
      Mode := ord(RadioButton2.Checked);
      IPAddress := IPAddressEdit.Text;
      try
        RemotePort := StrToInt(PortEdit.Text);
      except
      end;
      COMPort := UpDown1.Position;
      case BaudCombobox.ItemIndex of
        0: Baudrate := 9600;
        1: Baudrate := 19200;
        2: Baudrate := 38400;
        3: Baudrate := 115200;
      end;
      DigPinMode[2] := P2Combobox.ItemIndex;
      DigPinMode[3] := P3Combobox.ItemIndex;
      DigPinMode[4] := P4Combobox.ItemIndex;
      DigPinMode[5] := P5Combobox.ItemIndex;
      DigPinMode[6] := P6Combobox.ItemIndex;
      DigPinMode[7] := P7Combobox.ItemIndex;
      DigPinMode[8] := P8Combobox.ItemIndex;
      DigPinMode[9] := P9Combobox.ItemIndex;
      AnaPinMode[0] := A0Combobox.ItemIndex;
      AnaPinMode[1] := A1Combobox.ItemIndex;
      AnaPinMode[2] := A2Combobox.ItemIndex;
      AnaPinMode[3] := A3Combobox.ItemIndex;
      AnaPinMode[4] := A4Combobox.ItemIndex;
      AnaPinMode[5] := A5Combobox.ItemIndex;
    end;
  end;
  Dialog.Free;
end;

Procedure WriteToFile(AFileHandle:Word; D: PParameterStruct); export stdcall;
var i: integer;
    s: array[0..1000] of char;
begin
  with D^.UserDataPtr^ do begin
    StrPCopy(s, IntToStr(Mode) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IntToStr(COMPort) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IntToStr(Baudrate) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IPAddress + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IntToStr(RemotePort) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    for i:=2 to 9 do begin
      StrPCopy(s, IntToStr(DigPinMode[i]) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    end;
    for i:=0 to 5 do begin
      StrPCopy(s, IntToStr(AnaPinMode[i]) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    end;
  end;
end;

Procedure ReadFromFile(AFileHandle:Word; D: PParameterStruct); export stdcall;
var i: integer;
    s: array[0..1000] of char;
procedure ReadOneLine(FHandle:Word; Aps :PChar);
var i  : Integer;
begin
  i:=0;
  _lRead(FHandle,@Aps[i],1);
  repeat
    inc(i);
    _lRead(FHandle,@Aps[i],1);
  until (Aps[i-1]=#13) and (Aps[i]=#10);
  Aps[i-1]:=#0;
end;
begin
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.Mode := StrToInt(StrPas(s));
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.COMPort := StrToInt(StrPas(s));
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.Baudrate := StrToInt(StrPas(s));
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.IPAddress := StrPas(s);
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.RemotePort := StrToInt(StrPas(s));
  for i:=2 to 9 do begin
    ReadOneLine(AFileHandle, s); D^.UserDataPtr.DigPinMode[i] := StrToInt(StrPas(s));
  end;
  for i:=0 to 5 do begin
    ReadOneLine(AFileHandle, s); D^.UserDataPtr.AnaPinMode[i] := StrToInt(StrPas(s));
  end;
end;

procedure IsUserDLL32; export stdcall;
begin
end;

procedure IsDemoDLL; export stdcall;
begin
end;

exports
  GetParameterStruct,
  GetDialogEnableStruct,
  GetNumberOfInputsOutputs,
  CanSimulateDLL,
  InitSimulationDLL,
  SimulateDLL,
  InitUserDLL,
  DisposeUserDLL,
  EndSimulationDLL2,
  GetDLLName,
  CallParameterDialogDLL,
  WriteToFile,
  ReadFromFile,
  SetInputChar,
  SetOutputChar,
  IsDemoDLL,
  IsUserDLL32;


begin
  DecimalSeparator := '.';
  Application.UpdateFormatSettings := false;
  COMHandle := 0;
end.
