// Arduino - BORIS Communication via USB
//
// Commands:
//
// !xxmi  Configure pin xx as digital input  (xx: 01 ... 19)
// !xxmo  Configure pin xx as digital output
// !xxs0  Set pin xx to LOW
// !xxs1  Set pin xx to HIGH
// pwmxx  Configure pin xx as PWM output
// wyyyy  Set PWM output to yyyy (yyyy: 0000 ... 1023, command must follow pwmxx command!)
// ?xxm?  Get mode of pin xx (answer is "I" or "O")
// ?xxs?  Get state of pin xx (answer is "LOW" or "HIGH" für pin 1-13, 0 ... 1023 für analog inputs 14-19)

#include <SPI.h>  

// Global Variables
char orderChar[5];
int orderByte[5]; 
int pos = 0;
int pin = 0;
int currentPinState[20];
char currentPinMode[20];
boolean pwmActive = false;

int incomingByte = 0;
char incomingChar = ' ';

void setup()
{
  Serial.begin(9600);
}

void loop()
{
  if (Serial.available() > 0)
    {
      incomingByte = Serial.read();
      if (incomingByte > 13)
      {
        incomingChar = incomingByte;
        orderByte[pos] = incomingByte; 
        orderChar[pos] = incomingChar; 
        pos++;  
      }
    } 
  if (pos == 5)
    {
      parseCommand();  
    }
}

void selectPin()  
{
  if(orderChar[1] == '0')
    pin = orderByte[2] - 48;
  if(orderChar[1] == '1')
    pin = orderByte[2] - 38;
}

// Hauptprogramm
void parseCommand()
{
  // PWM
  if (pwmActive && orderChar[0] == 'w')
    {
      int e = orderByte[4] - 48;
      int z = (orderByte[3] - 48)*10;
      int h = (orderByte[2] - 48)*100;
      int t = (orderByte[1] - 48)*1000;
      int g = e+z+h+t;
      pwmActive = false;
      if (g >= 0 && g <= 1023)
      {
        analogWrite(pin, g);
        currentPinState[pin] = g;
        Serial.println("Set PWM Output");
      }
    }
  if(orderChar[0] == 'p' && orderChar[1] == 'w' && orderChar[2] == 'm')
    {
      if(orderChar[3] == '0')
        pin = orderByte[4] - 48;
      if(orderChar[3] == '1')
        pin = orderByte[4] - 38;
      pwmActive = true;
      Serial.println("Switched to PWM Mode");
    }
  // SET
  if(orderChar[0] == '!')
    {
      //Status setzen
      if(orderChar[3] == 's')
      {
        selectPin();
        if (orderChar[4] == '1')
         {
           digitalWrite(pin, HIGH);
           currentPinState[pin] = 1; 
           Serial.println("Set Pin to HIGH");
        } 
        if (orderChar[4] == '0')
         {
           digitalWrite(pin, LOW);
           currentPinState[pin] = 0; 
           Serial.println("Set Pin to LOW");
         }
      }          
      //Modus setzen
      if(orderChar[3] == 'm')
      {
        selectPin();
        if(orderChar[4] == 'o')
          {
            pinMode(pin, OUTPUT);
            currentPinMode[pin] = 'O';
            Serial.println("Set Pin to OUTPUT");
          }
       if(orderChar[4] == 'i')
         {
           pinMode(pin, INPUT);
           currentPinMode[pin] = 'I';  
           Serial.println("Set Pin to INPUT");
         }
      }
    } 
  
  // GET
  if(orderChar[0] == '?')
    {
      selectPin();
      if(orderChar[3] == 'm')
        {
          Serial.println(currentPinMode[pin]);
        }
      if(orderChar[3] == 's' && currentPinMode[pin] == 'O')
        {
          Serial.println(currentPinState[pin]);
       }
       if (orderChar[3] == 's' && currentPinMode[pin] == 'I')
         {
           if (pin<14)
             {
               int wert = digitalRead(pin);
               if (wert == LOW)
                 Serial.println("LOW");
               if (wert == HIGH)
                 Serial.println("HIGH");
             }
           if (pin >= 14)
             {
               int wert = analogRead(pin);
               Serial.println(wert);
             }
         }
    }
  pos = 0;  
}
