// Arduino - BORIS Communication via Ethernet (UDP)
//
// Commands:
//
// !xxmi  Configure pin xx as digital input  (xx: 01 ... 19)
// !xxmo  Configure pin xx as digital output
// !xxs0  Set pin xx to LOW
// !xxs1  Set pin xx to HIGH
// pwmxx  Configure pin xx as PWM output
// wyyyy  Set PWM output to yyyy (yyyy: 0000 ... 1023, command must follow pwmxx command!)
// ?xxm?  Get mode of pin xx (answer is "I" or "O")
// ?xxs?  Get state of pin xx (answer is "LOW" or "HIGH" für pin 1-13, 0 ... 1023 für analog inputs 14-19)

#include <SPI.h>  
#include <Ethernet.h>
#include <Udp.h>

// Global Variables
char orderChar[5];
int orderByte[5]; 
int pin = 0;
int currentPinState[20];
char currentPinMode[20];
boolean pwmActive = false;

int incomingByte = 0;
char incomingChar = ' ';

byte mac[] = {0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };  // MAC Address of Arduino
byte ip[] = {192,168,0,121};  // IP Address of Arduino
unsigned int localPort = 1600;  // Port
byte remoteIp[4];        // Remote IP-Address
unsigned int remotePort; // Remote Port
char packetBuffer[UDP_TX_PACKET_MAX_SIZE];


void setup()
{
  Ethernet.begin(mac, ip);
  Udp.begin(localPort);
}

void loop()
{
  int packetSize = Udp.available();
  if(packetSize)
  {
    packetSize = packetSize - 8;
    // read the packet into packetBufffer and get the senders IP addr and port number
    Udp.readPacket(packetBuffer,5, remoteIp, remotePort);
    for (int i = 0; i < 5; i++)
    {
      orderChar[i] = packetBuffer[i];
      orderByte[i] = orderChar[i];
    }
    parseCommand();  
  }
  delay(10);
}

void selectPin()  
{
  if(orderChar[1] == '0')
    pin = orderByte[2] - 48;
  if(orderChar[1] == '1')
    pin = orderByte[2] - 38;
}

void parseCommand()
{
  // PWM
  if (pwmActive && orderChar[0] == 'w')
    {
      int e = orderByte[4] - 48;
      int z = (orderByte[3] - 48)*10;
      int h = (orderByte[2] - 48)*100;
      int t = (orderByte[1] - 48)*1000;
      int g = e+z+h+t;
      pwmActive = false;
      if (g >= 0 && g <= 1023)
      {
        analogWrite(pin, g);
        currentPinState[pin] = g;
        Udp.sendPacket("Set PWM Output", remoteIp, remotePort);
      }
    }
  if(orderChar[0] == 'p' && orderChar[1] == 'w' && orderChar[2] == 'm')
    {
      if(orderChar[3] == '0')
        pin = orderByte[4] - 48;
      if(orderChar[3] == '1')
        pin = orderByte[4] - 38;
      pwmActive = true;
      Udp.sendPacket("Switched to PWM Mode", remoteIp, remotePort);
    }
  // SET
  if(orderChar[0] == '!')
    {
      //Status setzen
      if(orderChar[3] == 's')
      {
        selectPin();
        if (orderChar[4] == '1')
         {
           digitalWrite(pin, HIGH);
           currentPinState[pin] = 1; 
           Udp.sendPacket("Set Pin to HIGH", remoteIp, remotePort);
        } 
        if (orderChar[4] == '0')
         {
           digitalWrite(pin, LOW);
           currentPinState[pin] = 0; 
           Udp.sendPacket("Set Pin to LOW", remoteIp, remotePort);
         }
      }          
      //Modus setzen
      if(orderChar[3] == 'm')
      {
        selectPin();
        if(orderChar[4] == 'o')
          {
            pinMode(pin, OUTPUT);
            currentPinMode[pin] = 'O';
            Udp.sendPacket("Set Pin to OUTPUT", remoteIp, remotePort);
          }
       if(orderChar[4] == 'i')
         {
           pinMode(pin, INPUT);
           currentPinMode[pin] = 'I';  
           Udp.sendPacket("Set Pin to INPUT", remoteIp, remotePort);
         }
      }
    } 
  
  // GET
  if(orderChar[0] == '?')
    {
      selectPin();
      if(orderChar[3] == 'm')
        {
          if (currentPinMode[pin] == 'O')
            Udp.sendPacket("O", remoteIp, remotePort);
          if (currentPinMode[pin] == 'I')
            Udp.sendPacket("I", remoteIp, remotePort);
        }
      if(orderChar[3] == 's' && currentPinMode[pin] == 'O')
        {
          if (currentPinState[pin] == 0)
            Udp.sendPacket("0", remoteIp, remotePort);
          if (currentPinState[pin] == 1)
            Udp.sendPacket("1", remoteIp, remotePort);
       }
       if (orderChar[3] == 's' && currentPinMode[pin] == 'I')
         {
           if (pin<14)
             {
               int wert = digitalRead(pin);
               if (wert == LOW)
                 Udp.sendPacket("LOW", remoteIp, remotePort);
               if (wert == HIGH)
                 Udp.sendPacket("HIGH", remoteIp, remotePort);
             }
           if (pin >= 14)
             {
               char s[4];
               itoa(analogRead(pin), s, 10);
               Udp.sendPacket(s, remoteIp, remotePort);
             }
         }
    }
}
